//+------------------------------------------------------------------+
//|                                              SuperWoodiesCCI.mq4 |
//|                                                           duckfu |
//|                                          http://www.dopeness.org |
//+------------------------------------------------------------------+
#property copyright "duckfu"
#property link      "http://www.dopeness.org"
//----
#property indicator_separate_window
#property indicator_buffers 5
#property indicator_color1 White // CCI Colour (note the superfluous vowel, cheers mates ;)
#property indicator_color2 Yellow // TCCI Colour
#property indicator_color3 LimeGreen // Long Trend Histogram Colour 
#property indicator_color4 Red // Short Trend Histogram Colour
#property indicator_color5 Blue // No Trend Histogram Colour
//---- input parameters
extern string TimeFrame="Current time frame";
extern int    CCI_Period=50;
extern int    TCCI_Period=0;
extern bool   Interpolate=true;
//---- buffers
double ExtCCIBuffer[];
double ExtTCCIBuffer[];
double ExtLongHistBuffer[];
double ExtShortHistBuffer[];
double ExtFlatHistBuffer[];

//
//
//
//
//

int    timeFrame;
string IndicatorFileName;
bool   ReturningBars;
bool   CalculatingCci;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   SetIndexBuffer(0,ExtCCIBuffer);
   SetIndexBuffer(1,ExtTCCIBuffer);
   SetIndexBuffer(2,ExtLongHistBuffer);  SetIndexStyle(2,DRAW_HISTOGRAM,STYLE_SOLID,1);
   SetIndexBuffer(3,ExtShortHistBuffer); SetIndexStyle(3,DRAW_HISTOGRAM,STYLE_SOLID,1);
   SetIndexBuffer(4,ExtFlatHistBuffer);  SetIndexStyle(4,DRAW_HISTOGRAM,STYLE_SOLID,1);
  
   //
   //
   //
   //
   //
         
   IndicatorFileName   = WindowExpertName();
   CalculatingCci = (TimeFrame=="CalculateCci"); if (CalculatingCci) return(0);
   ReturningBars  = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
   timeFrame      = stringToTimeFrame(TimeFrame);
   IndicatorShortName("Super Woodie CCI "+timeFrameToString(timeFrame)+" ("+CCI_Period+","+TCCI_Period+")");
   return(0);
}

//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
  int deinit()
  {
//---- TODO: add your code here
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
  int start()
  {
   int i,j,limit,counted_bars=IndicatorCounted();
   int uptrending=0,downtrending=0;
//---- TODO: add your code here
   if(counted_bars<0) return(-1);
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=MathMin(Bars-counted_bars,Bars-1);
           if (ReturningBars)  { ExtCCIBuffer[0] = limit; return(0); }
   
//---- main loop
   //
   //
   //
   //
   //
   
   if (CalculatingCci || timeFrame==Period())
   {
     for(i=0; i<limit; i++)
     {
      ExtCCIBuffer[i]=iCCI(NULL,0,CCI_Period,PRICE_TYPICAL,i);
      ExtTCCIBuffer[i]=iCCI(NULL,0,TCCI_Period,PRICE_TYPICAL,i);
      ExtFlatHistBuffer[i]=ExtCCIBuffer[i];
      ExtShortHistBuffer[i]=0;
      ExtLongHistBuffer[i]=0;
     }
     for(i=0;i<limit;i++)
     {
        for(j=0;j<6;j++)
        {
           if(ExtCCIBuffer[i+j] > 0)
           {
            uptrending++;
            } else if(ExtCCIBuffer[i+j] < 0)
               {
               uptrending=0;
               }
        }
        if(uptrending>5)
        {
         ExtLongHistBuffer[i]=ExtCCIBuffer[i];
         ExtShortHistBuffer[i]=0;
         ExtFlatHistBuffer[i]=0;
        }
     }
     for(i=0;i<limit;i++)
     {
        for(j=0;j<6;j++)
        {
           if(ExtCCIBuffer[i+j] < 0)
           {
            downtrending++;
            }
             else if(ExtCCIBuffer[i+j] > 0)
            {
               downtrending=0;
            }
        }
        if(downtrending>5)
        {
         ExtShortHistBuffer[i]=ExtCCIBuffer[i];
         ExtLongHistBuffer[i]=0;
         ExtFlatHistBuffer[i]=0;
        }
     }
     return(0);
   }
   
   //
   //
   //
   //
   //     
   
   if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         ExtCCIBuffer[i]       = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateCci",CCI_Period,TCCI_Period,0,y);
         ExtTCCIBuffer[i]      = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateCci",CCI_Period,TCCI_Period,1,y);
         ExtLongHistBuffer[i]  = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateCci",CCI_Period,TCCI_Period,2,y);
         ExtShortHistBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateCci",CCI_Period,TCCI_Period,3,y);
         ExtFlatHistBuffer[i]  = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateCci",CCI_Period,TCCI_Period,4,y);

         //
         //
         //
         //
         //
      
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(int k = 1; k < n; k++)
            {
               ExtCCIBuffer[i+k]       = k*factor*ExtCCIBuffer[i+n]       + (1.0-k*factor)*ExtCCIBuffer[i];
               ExtTCCIBuffer[i+k]      = k*factor*ExtTCCIBuffer[i+n]      + (1.0-k*factor)*ExtTCCIBuffer[i];
               ExtLongHistBuffer[i+k]  = k*factor*ExtLongHistBuffer[i+n]  + (1.0-k*factor)*ExtLongHistBuffer[i];
               ExtShortHistBuffer[i+k] = k*factor*ExtShortHistBuffer[i+n] + (1.0-k*factor)*ExtShortHistBuffer[i];
               ExtFlatHistBuffer[i+k]  = k*factor*ExtFlatHistBuffer[i+n]  + (1.0-k*factor)*ExtFlatHistBuffer[i];
            }               
   }
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}